/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2013-2023 Chaosdorf e.V.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package de.chaosdorf.meteroid.ui.money

import androidx.annotation.DrawableRes
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import dagger.hilt.android.lifecycle.HiltViewModel
import de.chaosdorf.mete.model.MeteApiFactory
import de.chaosdorf.meteroid.R
import de.chaosdorf.meteroid.model.AccountInfo
import de.chaosdorf.meteroid.sync.AccountProvider
import de.chaosdorf.meteroid.ui.SyncManager
import kotlinx.coroutines.flow.SharingStarted
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.flow.stateIn
import kotlinx.coroutines.launch
import java.math.BigDecimal
import javax.inject.Inject

enum class MonetaryAmount(val amount: BigDecimal, @DrawableRes val image: Int) {
  MONEY_50(0.50.toBigDecimal(), R.drawable.euro_50),
  MONEY_100(1.00.toBigDecimal(), R.drawable.euro_100),
  MONEY_200(2.00.toBigDecimal(), R.drawable.euro_200),
  MONEY_500(5.00.toBigDecimal(), R.drawable.euro_500),
  MONEY_1000(10.00.toBigDecimal(), R.drawable.euro_1000),
  MONEY_2000(20.00.toBigDecimal(), R.drawable.euro_2000),
  MONEY_5000(50.00.toBigDecimal(), R.drawable.euro_5000),
}

@HiltViewModel
class MoneyListViewModel @Inject constructor(
  accountProvider: AccountProvider,
  private val syncManager: SyncManager
) : ViewModel() {
  val account: StateFlow<AccountInfo?> = accountProvider.account
    .stateIn(viewModelScope, SharingStarted.WhileSubscribed(), null)

  val money: List<MonetaryAmount> = MonetaryAmount.entries

  fun deposit(item: MonetaryAmount) {
    account.value?.let { account ->
      viewModelScope.launch {
        syncManager.deposit(account, item.amount)
      }
    }
  }
}
